from __future__ import absolute_import
from __future__ import print_function
import os, time, subprocess, signal, re, sys

ON_WINDOWS = (sys.platform == 'win32')
SIGNALS = {
    signal.SIGINT: 'SIGINT',
    signal.SIGTERM: 'SIGTERM',
}
if ON_WINDOWS:
    SIGNALS[signal.SIGBREAK] = 'SIGBREAK'

child = None

def term(proc):
    """Send a SIGTERM/SIGBREAK to *proc* and wait for it to terminate."""
    if ON_WINDOWS:
        proc.send_signal(signal.CTRL_BREAK_EVENT)
    else:
        proc.terminate()
    proc.wait()
    
def sigHandler(signum, frame):
    """Stop the sub-process *child* if *signum* is SIGTERM. Then terminate."""
    try:
        print(('received signal=%s' % (SIGNALS[signum])))
        if child and (ON_WINDOWS or signum != signal.SIGINT):
            # Forward SIGTERM on Linux or any signal on Windows
            print(('forwarding received signal=%s to child' % (SIGNALS[signum])))
            term(child)
    except:
        traceback.print_exc()
    finally:
        sys.exit()
  
def subproc(args, env=None):
    """Create and return a new subprocess named *name*."""
    kwargs = {}
    if ON_WINDOWS:
        kwargs['creationflags'] = subprocess.CREATE_NEW_PROCESS_GROUP
    else:
        kwargs['preexec_fn'] = os.setsid
    proc = subprocess.Popen(args, env=env, **kwargs)
    return proc


def runCriblCommand(args):
    splunk_home = os.getenv("SPLUNK_HOME", "")
    # run Cribl command ... start it as a separate process, poll its status periodically, that way 
    # this process is able to receive and propagate signals to it. It sucks that os.exec doesn't do the right thing in windows, ohh well!
    signal.signal(signal.SIGINT, sigHandler)
    if ON_WINDOWS:
        signal.signal(signal.SIGBREAK, sigHandler)
    else:
        signal.signal(signal.SIGTERM, sigHandler)

    cribl_cmd = [os.path.join(splunk_home, "etc", "apps", "cribl", "bin", "cribl")] + args
    cmd_env = os.environ.copy()
    cmd_env["CRIBL_PY"] = "1"
    global child
    child = subproc(cribl_cmd, cmd_env)
    while True:
        time.sleep(0.1)
        retcode = child.poll()
        if retcode is not None:
            sys.exit(retcode)

if __name__ == "__main__":
    runCriblCommand(sys.argv[1:])